#!/usr/bin/python3

import requests
import RPi.GPIO as GPIO
import time
import calendar
import signal

time.sleep(15) # allow for network to come up after boot...

GPIO.setmode(GPIO.BOARD)
GPIO.setup(13,GPIO.OUT, initial=GPIO.LOW) # Decimal 10^5
GPIO.setup(12,GPIO.OUT, initial=GPIO.LOW) # Reset
GPIO.setup(16,GPIO.OUT, initial=GPIO.LOW) # 1K -35V
GPIO.setup(22,GPIO.OUT, initial=GPIO.LOW) # 1K  GRD
GPIO.setup(33,GPIO.OUT, initial=GPIO.LOW) # Decimal 10^1
GPIO.setup(29,GPIO.OUT, initial=GPIO.LOW) # Decimal 10^3
GPIO.setup(36,GPIO.OUT, initial=GPIO.LOW) # Decimal 10^4
GPIO.setup(32,GPIO.OUT, initial=GPIO.LOW) # Decimal 10^2
GPIO.setup(7,GPIO.OUT, initial=GPIO.LOW) # Decimal 10^6
GPIO.setup(18,GPIO.OUT, initial=GPIO.LOW) # Count line

# Below copied from StackExchange
class GracefulKiller:
  kill_now = False
  def __init__(self):
    signal.signal(signal.SIGINT, self.exit_gracefully)
    signal.signal(signal.SIGTERM, self.exit_gracefully)

  def exit_gracefully(self,signum, frame):
    self.kill_now = True

#-----------------------------End of copy

def reset():
	GPIO.output(12,GPIO.HIGH)
	time.sleep(.1)
	GPIO.output(12,GPIO.LOW)
	time.sleep(.1)

def show_trend(v):
	if v>1000:
		v-=2000
		selftest=True
		t_show=5
	else:
		selftest=False
		t_show=30
	if v==999:
		speed=.2
		t_end = time.time() + t_show # run for 30 seconds from 'now'
		while time.time() < t_end:
			GPIO.output(29,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(29,GPIO.LOW)
			GPIO.output(36,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(36,GPIO.LOW)
			GPIO.output(7,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(7,GPIO.LOW)
			GPIO.output(32,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(32,GPIO.LOW)
			GPIO.output(13,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(13,GPIO.LOW)
			GPIO.output(33,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(33,GPIO.LOW)
		return
	if v<=0:
		upTrend=False
		v=abs(v)
	else:
		upTrend=True
	c=0
	if 3 <= v < 5:
		speed=.2
	elif 2 <= v < 3:
		speed=.3
	elif  1 <= v <2:
		speed=.4
	elif 5 <= v <= 7:
		speed=.15
	elif 7 < v <=10:
		speed=.1
	elif v>10:
		speed=.05
	elif v<1:
		speed=.7
	print("speed is {}".format(speed))
	print("uptrend is {}".format(upTrend))
	t_end = time.time() + t_show # run for 30 seconds from 'now'
	while time.time() < t_end:
		if upTrend: # go left,we're going up
			GPIO.output(29,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(29,GPIO.LOW)
			GPIO.output(32,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(32,GPIO.LOW)
			GPIO.output(33,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(33,GPIO.LOW)
			GPIO.output(36,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(36,GPIO.LOW)
			GPIO.output(13,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(13,GPIO.LOW)
			GPIO.output(7,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(7,GPIO.LOW)
		else:
			GPIO.output(7,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(7,GPIO.LOW)
			GPIO.output(13,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(13,GPIO.LOW)
			GPIO.output(36,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(36,GPIO.LOW)
			GPIO.output(33,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(33,GPIO.LOW)
			GPIO.output(32,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(32,GPIO.LOW)
			GPIO.output(29,GPIO.HIGH)
			time.sleep(speed)
			GPIO.output(29,GPIO.LOW)



def count_1k(v):
	global isReset
	c=0
	if v>=3:
		v-=1   #do one less because it counts an extra one if the counter has not just been reset
	while(c<=v):
		GPIO.output(22,GPIO.HIGH)
		time.sleep(.012)
		GPIO.output(22,GPIO.LOW)
		if v==1 and not isReset:
			print("V is 1 and isReset is False")
			return
		time.sleep(.0001)
		GPIO.output(16,GPIO.HIGH)
		time.sleep(.012)
		GPIO.output(16,GPIO.LOW)
		time.sleep(.0001)
		c+=1

def count_display(v):
	c=0
	while(c<v):
		GPIO.output(18,GPIO.HIGH)
		if c<5:
			time.sleep(.01)
		GPIO.output(18,GPIO.LOW)
		if c<5:
			time.sleep(.01)
		c+=1

def getsgv():
	response=requests.get('http://192.168.84.246:1337/api/v1/entries/sgv')
	payload=response.text.split()
	global sgv
	global weightedAvg
	LastRead=(payload[0])
	pat='%Y-%m-%dT%H:%M:%S.%Z'
	timeOfLastRead=time.strptime(LastRead[1:-5]+"GMT",pat) # strip leading quote and trailing decimal of seconds, add GMT to end
	epoch_timeOfLastRead=calendar.timegm(timeOfLastRead)
	sinceLast=(time.time())-epoch_timeOfLastRead
	print("sincelast is {}".format(sinceLast))
	sgv=int(payload[2])
	LastSGV=int(payload[7])
	SecondLastSGV=int(payload[12])
	ThirdLastSGV=int(payload[17])
	delta1=sgv-LastSGV
	delta2=LastSGV-SecondLastSGV
	delta3=SecondLastSGV-ThirdLastSGV
	tempvar=delta3+(delta2*1.5)+(delta1*3)
	weightedAvg=tempvar/5.5
	if sinceLast>900:
		weightedAvg=999
	print("weightedAvg is {}".format(weightedAvg))
	print("{}, {}, {}".format(sgv, LastSGV, SecondLastSGV))

def getTime():
	ctime=time.localtime(time.time())
	h=ctime[3]
	if h==0:
		h=12
	if h>12:
		h=h-12
	h=str(h).zfill(2)
	m=str(ctime[4]).zfill(2)
	s=str(ctime[5]).zfill(2)
	currentTime=h+m+s
	return currentTime

def convTime(t):
	if t<59:
		return t
	tempValue=int(str(t)[-2:])
	if tempValue>59:
		print("{} is greater than 59".format(t))
		t=(t+40)
	else:
		print("{} is less than 59".format(t))
	return t


def displayTime(v):
	global isReset
	if v>1000:
		v-=1000
		selftest=True
	else:
		selftest=False
	currentTime=getTime()
	first3=currentTime[:3]
	last3=currentTime[-3:]
	first3=int(first3)
	last3=int(last3)
	reset()
	isReset=True
	count_display(last3)
	count_1k(first3)
	isReset=False
	GPIO.output(13,GPIO.HIGH)
	GPIO.output(33,GPIO.HIGH)
	if selftest:
		c=-30
	else:
		c=0
	while c<=30:
		time.sleep(1)
		if int(str(last3)[-2:])==59:
			if int(str(last3)[:1])==9:
				if int(str(first3)[-1:])==5:
					count_1k(5)
					first3+=5
				else:
					count_1k(1)
					first3+=1
			count_display(41)
			last3+=41
		else:
			count_display(1)
			last3+=1
		c+=1
	GPIO.output(13,GPIO.LOW)
	GPIO.output(33,GPIO.LOW)


def selftest():
	reset()
	isReset=True
	getsgv()
	count_display(sgv)
	show_trend(2999)
	show_trend(1990)
	show_trend(1996)
	show_trend(2000)
	show_trend(2004)
	show_trend(2010)
	displayTime(sgv)

killer = GracefulKiller()

while not killer.kill_now:
	reset()
	isReset=True
	getsgv()
	count_display(sgv)
	show_trend(weightedAvg)
	displayTime(sgv)


reset()
count_display(843)
GPIO.cleanup()
